#include "OneWireHub.h"
#include "DS2408.h"

const bool dbg_DS2408 = 0; // give debug messages for this sensor

//=================== DS2408 ==========================================
DS2408::DS2408(uint8_t ID1, uint8_t ID2, uint8_t ID3, uint8_t ID4, uint8_t ID5, uint8_t ID6, uint8_t ID7) : OneWireItem(ID1, ID2, ID3, ID4, ID5, ID6, ID7)
{
    this->memory[0] = 0xF0;  // Cmd
    this->memory[1] = 0x88;  // AdrL
    this->memory[2] = 0x00;  // AdrH
    this->memory[3] = 0;     // D0
    this->memory[4] = 0;     // D1
    this->memory[5] = 0;     // D2
    this->memory[6] = 0;     // D3
    this->memory[7] = 0;     // D4
    this->memory[8] = 0;     // D5
    this->memory[9] = 0xFF;  // D6
    this->memory[10] = 0xFF; // D7
    this->memory[11] = 0;  // CRCL
    this->memory[12] = 0;  // CRCH
}

bool DS2408::updateCRC()
{
    ow_crc16_reset();
    for (int i = 0; i < 11; i++) ow_crc16_update(this->memory[i]);
    ((sDS2408 *) (this->memory))->CRC = ow_crc16_get();
}

bool DS2408::duty(OneWireHub *hub)
{
    uint8_t addrL;
    uint8_t addrH;
    uint8_t data;

    uint8_t done = hub->recv();

    switch (done)
    {
        // Read PIO Registers
        case 0xF0:
            // Cmd
            this->memory[0] = done;

            // AdrL
            this->memory[1] = hub->recv();

            // AdrH
            this->memory[2] = hub->recv();

            // UpdateCrc
            this->updateCRC();

            // Data
            data = hub->sendData(this->memory + 3, 10);

            if (dbg_DS2408)
            {
                Serial.print("DS2408 : PIO Registers : ");
                Serial.print(this->memory[2], HEX);
                Serial.print(" ");
                Serial.print(this->memory[1], HEX);
                Serial.println();
            }

            break;

        default:
            if (dbg_HINT)
            {
                Serial.print("DS2408=");
                Serial.println(done, HEX);
            }
            return FALSE;
    }

    return TRUE;
}
